#include <avr/io.h>
#include <avr/interrupt.h>

#include <stdbool.h>

// The UART is on PD2 (Rx) and PD3 (Tx)
// The gamepads are all connected on port B
#define PCW_PORT PORTD
#define PCW_DDR  DDRD
#define PCW_PIN  PIND
#define PCW_CLK_A  (1 << 0)
#define PCW_DAT_A  (1 << 1)

uint8_t matrix[16];

static uint16_t scanbyte()
{
	uint8_t pulses = 0;
	uint8_t oldv, newv;
	uint16_t rxbuffer = 0;

	// This is a bit timing critical so don't be interrupted while we do it
	cli();
	oldv = PCW_PIN;

	for (;;) {
		newv = PCW_PIN;

		if (pulses < 2) {
			// Initial sync phase: 2 pulses on data without changes of clock
			if (((newv & PCW_CLK_A) == 0) && ((oldv & PCW_CLK_A) != 0)) {
				// Clock moved while we were waiting for the initial data pulses
				// We are not in sync, start over
				pulses = 0;
				rxbuffer = 0;
			}
			if (((newv & PCW_DAT_A) == 0) && ((oldv & PCW_DAT_A) != 0)) {
				pulses++;
			}
		} else {
			// Receiving data (8 bits)
			if (((newv & PCW_CLK_A) == 0) && ((oldv & PCW_CLK_A) != 0)) {
				rxbuffer = (rxbuffer << 1);
				if (newv & PCW_DAT_A)
					rxbuffer |= 1;
				pulses++;

				// If we reached the last bit, we can stop
				if (pulses == 14) {
					// We can be interrupted again.
					sei();
					return rxbuffer;
				}
			}
		}

		oldv = newv;
	}
}

extern "C" void scanall()
{
	uint16_t rxbuffer;
	
	for (int i = 0; i < 17; i++) {
		rxbuffer = scanbyte();
		matrix[rxbuffer >> 8] = rxbuffer & 0xFF;
	}
}

extern "C" void PCW_Init() {
	// GrIP input
	PCW_PORT |= 0x3; // Enable pull-ups
	PCW_DDR &= 0xFC;  // Port as input
}

